@@ -2,7 +2,7 @@
 #	E-mail:		drpython@bluebottle.com
 #	Note:		You must reply to the verification e-mail to get through.
 #
-#	Copyright 2003-2004 Daniel Pozmanter
+#	Copyright 2003-2005 Daniel Pozmanter
 #
 #	Distributed under the terms of the GPL (GNU Public License)
 #
@@ -19,7 +19,7 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-	
+
 #ToolBar File
 
 import os.path
@@ -27,14 +27,14 @@ import drPrefsFile
 import wx
 import drScrolledMessageDialog
 
-def getToolBarList(homedirectory):
-	toolbarfile = homedirectory + "/toolbar.dat"
+def getToolBarList(userpreferencesdirectory):
+	toolbarfile = userpreferencesdirectory + "/toolbar.dat"
 	if not os.path.exists(toolbarfile):
 		return ["New", "Open", "Reload File", "Close", "Save", \
 		"<Separator>", "Print File", "Print Prompt", \
 		"<Separator>", "Find", "Replace", \
 		"<Separator>", "<Separator>", "Preferences", "Toggle Prompt", "Toggle View Whitespace", \
-		"<Separator>", "<Separator>", "<Separator>", "Run", "Set Arguments", "Python", \
+		"<Separator>", "<Separator>", "<Separator>", "Check Syntax", "Run", "Set Arguments", "Python", \
 		"End"]
 	ToolBarList = []
 	f = file(toolbarfile, 'r')
@@ -42,18 +42,18 @@ def getToolBarList(homedirectory):
 	while (len(line) > 0):
 		ToolBarList.append(line)
 		line = f.readline().rstrip()
-	f.close()	
+	f.close()
 	return ToolBarList
 
-def getCustomBitmaps(homedirectory):
-	filename = homedirectory + "/toolbar.custom.icons.dat"
+def getCustomBitmaps(userpreferencesdirectory):
+	filename = userpreferencesdirectory + "/toolbar.custom.icons.dat"
 	if not os.path.exists(filename):
 		return [], [], []
-		
+
 	nameArray = []
 	fileArray16 = []
 	fileArray24 = []
-	
+
 	f = file(filename, 'r')
 	line = f.readline()
 	while (len(line) > 0):
@@ -62,65 +62,31 @@ def getCustomBitmaps(homedirectory):
 		fileArray24.append(drPrefsFile.ExtractPreferenceFromText(line, '24'))
 		line = f.readline()
 	f.close()
-	
+
 	return nameArray, fileArray16, fileArray24
 
-def getBitmapFileArray(filename, bitmapNames, defaultbitmapdirectory, iconsize, customNames, customFiles):
+def getBitmapFileArray(bitmapNames, defaultbitmapdirectory, iconsize, customNames, customFiles):
 	iconsizestr = str(iconsize)
 	fileArray = []
-		
-	useDefault = (len(filename) < 0)
-	
-	if not useDefault:
-		try:
-			f = file(filename, 'r')
-			text = f.read()
-			f.close()
-			basepath = os.path.split(filename)[0]
-			if (not os.path.exists(basepath + "/" + iconsizestr)):
-				useDefault = 1
-			else:							
-				t = drPrefsFile.ExtractPreferenceFromText(text, "Default")
-				icondefault = ""
-				if len(t) > 0:		
-					icondefault = basepath + "/" + iconsizestr + "/" + t
-				if (len(t) < 1) or not os.path.exists(icondefault):
-					icondefault = defaultbitmapdirectory + "/" + iconsizestr + "/Default.png"
-		except:
-			useDefault = 1
-	
+
 	l = len(bitmapNames)
 	x = 0
 	while (x < l):
 		if not bitmapNames[x] == "<Separator>":
-			if (useDefault):	
-				if bitmapNames[x] in customNames:
-					i = customNames.index(bitmapNames[x])
-					t = customFiles[i]
-					if not os.path.exists(t):
-						t = defaultbitmapdirectory + "/" + iconsizestr + "/" + bitmapNames[x] + ".png"
-				else:
+			if bitmapNames[x] in customNames:
+				i = customNames.index(bitmapNames[x])
+				t = customFiles[i]
+				if not os.path.exists(t):
 					t = defaultbitmapdirectory + "/" + iconsizestr + "/" + bitmapNames[x] + ".png"
-				if not os.path.exists(t):					
-					t = defaultbitmapdirectory + "/" + iconsizestr + "/Default.png"			
 			else:
-				t = ""
-				if bitmapNames[x] in customNames:
-					i = customNames.index(bitmapNames[x])
-					t = customFiles[i]
-				else:
-					t = drPrefsFile.ExtractPreferenceFromText(text, bitmapNames[x])
-					if (len(t) < 1):
-						t = icondefault
-					else:
-						t = basepath + "/" + iconsizestr + "/" + t
-				if (len(t) < 1) or (not os.path.exists(t)):
-					t = icondefault
+				t = defaultbitmapdirectory + "/" + iconsizestr + "/" + bitmapNames[x] + ".png"
+			if not os.path.exists(t):
+				t = defaultbitmapdirectory + "/" + iconsizestr + "/Default.png"
 			fileArray.append(t)
 		else:
 			fileArray.append("<Separator>")
 		x = x + 1
-		
+
 	return fileArray
 
 def AddandReturn(frame, bitmapFileArray, IDNUM, count, title = ""):
@@ -132,20 +98,18 @@ def AddandReturn(frame, bitmapFileArray, IDNUM, count, title = ""):
 
 def SetupToolBar(frame):
 	try:
-		names, f16, f24 = getCustomBitmaps(frame.homedirectory)
-		
+		names, f16, f24 = getCustomBitmaps(frame.userpreferencesdirectory)
+
 		if frame.prefs.iconsize == 16:
-			bitmapFileArray = getBitmapFileArray(frame.prefs.iconfile, \
-			frame.ToolBarList, frame.bitmapdirectory, frame.prefs.iconsize, names, f16)
+			bitmapFileArray = getBitmapFileArray(frame.ToolBarList, frame.bitmapdirectory, frame.prefs.iconsize, names, f16)
 		else:
-			bitmapFileArray = getBitmapFileArray(frame.prefs.iconfile, frame.ToolBarList, \
-			frame.bitmapdirectory, frame.prefs.iconsize, names, f24)
-	
-				
+			bitmapFileArray = getBitmapFileArray(frame.ToolBarList, frame.bitmapdirectory, frame.prefs.iconsize, names, f24)
+
+
 		frame.toolbar.SetToolBitmapSize(wx.Size(frame.prefs.iconsize, frame.prefs.iconsize))
-		
+
 		ToolBarIdList = []
-		
+
 		x = 0
 		l = len(frame.ToolBarList)
 		while x < l:
@@ -177,7 +141,7 @@ def SetupToolBar(frame):
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_CLOSE_ALL, x))
 			elif frame.ToolBarList[x] == "Close All Other Documents":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_CLOSE_ALL_OTHER_TABS, x))
-				
+
 			elif frame.ToolBarList[x] == "Print Setup":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_PRINT_SETUP, x))
 			elif frame.ToolBarList[x] == "Print File":
@@ -187,7 +151,7 @@ def SetupToolBar(frame):
 
 			elif frame.ToolBarList[x] == "Exit":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_EXIT, x))
-	
+
 			elif frame.ToolBarList[x] == "Next Document":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_NEXT_TAB, x))
 			elif frame.ToolBarList[x] == "Previous Document":
@@ -207,11 +171,30 @@ def SetupToolBar(frame):
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_FIND_PREVIOUS, x))
 			elif frame.ToolBarList[x] == "Insert Regular Expression":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_INSERT_REGEX, x))
+			elif frame.ToolBarList[x] == "Insert Separator":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_INSERT_SEPARATOR, x))
+
 			elif frame.ToolBarList[x] == "Go To":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_GOTO, x))
+			elif frame.ToolBarList[x] == "Go To Block Start":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_GOTO_BLOCK_START, x))
+			elif frame.ToolBarList[x] == "Go To Block End":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_GOTO_BLOCK_END, x))
+			elif frame.ToolBarList[x] == "Go To Class Start":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_GOTO_CLASS_START, x))
+			elif frame.ToolBarList[x] == "Go To Class End":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_GOTO_CLASS_END, x))
+			elif frame.ToolBarList[x] == "Go To Def Start":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_GOTO_DEF_START, x))
+			elif frame.ToolBarList[x] == "Go To Def End":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_GOTO_DEF_END, x))
+
+			elif frame.ToolBarList[x] == "Source Browser Go To":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_SOURCEBROWSER_GOTO, x))
+
 			elif frame.ToolBarList[x] == "Find And Complete":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_FIND_AND_COMPLETE, x))
-				
+
 			elif frame.ToolBarList[x] == "Comment":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_COMMENT_REGION, x))
 			elif frame.ToolBarList[x] == "UnComment":
@@ -228,10 +211,10 @@ def SetupToolBar(frame):
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_UPPERCASE, x))
 			elif frame.ToolBarList[x] == "Lowercase":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_LOWERCASE, x))
-				
+
 			elif frame.ToolBarList[x] == "Find And Complete":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_FIND_AND_COMPLETE, x))
-				
+
 			elif frame.ToolBarList[x] == "Dynamic DrScript":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.drscriptmenu.ID_DYNAMIC_SCRIPT, x))
 
@@ -252,45 +235,43 @@ def SetupToolBar(frame):
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_ZOOM_IN, x))
 			elif frame.ToolBarList[x] == "Zoom Out":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_ZOOM_OUT, x))
+			elif frame.ToolBarList[x] == "Toggle Fold":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_TOGGLE_FOLD, x))
 			elif frame.ToolBarList[x] == "Fold All":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_FOLD_ALL, x))
 			elif frame.ToolBarList[x] == "Expand All":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_EXPAND_ALL, x))
-				
+
 			elif frame.ToolBarList[x] == "Select All":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_SELECT_ALL, x))
-						
+
 			elif frame.ToolBarList[x] == "View In Left Panel":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_VIEW_IN_LEFT_PANEL, x))
 			elif frame.ToolBarList[x] == "View In Right Panel":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_VIEW_IN_RIGHT_PANEL, x))
 			elif frame.ToolBarList[x] == "View In Top Panel":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_VIEW_IN_TOP_PANEL, x))
-			elif frame.ToolBarList[x] == "View In Bottom Panel":
-				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_VIEW_IN_BOTTOM_PANEL, x))
-				
+
 			elif frame.ToolBarList[x] == "Toggle Source Browser":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_TOGGLE_SOURCEBROWSER, x))
 			elif frame.ToolBarList[x] == "Preferences":
 			#Fixed by limodou:
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_PREFS, x))
 			#End limodou
-			
+
 			elif frame.ToolBarList[x] == "Customize Shortcuts":
-				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_SHORTCUTS, x))			
-				
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_SHORTCUTS, x))
+
 			elif frame.ToolBarList[x] == "Customize Pop Up Menu":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_POPUP, x))
-			
+
 			elif frame.ToolBarList[x] == "Customize ToolBar":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_CUSTOMIZE_TOOLBAR, x))
-			
+
 			elif frame.ToolBarList[x] == "Toggle Prompt":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_TOGGLE_PROMPT, x))
 			elif frame.ToolBarList[x] == "Toggle View Whitespace":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_TOGGLE_VIEWWHITESPACE, x))
-			elif frame.ToolBarList[x] == "Clear Prompt":
-				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_CLEAR_PROMPT, x))
 
 			elif frame.ToolBarList[x] == "Run":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_RUN, x))
@@ -300,6 +281,10 @@ def SetupToolBar(frame):
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_PYTHON, x))
 			elif frame.ToolBarList[x] == "End":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_END, x))
+			elif frame.ToolBarList[x] == "Check Syntax":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_CHECK_SYNTAX, x))
+			elif frame.ToolBarList[x] == "Close Prompt":
+				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_CLOSE_PROMPT, x))
 
 			elif frame.ToolBarList[x] == "Help":
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_HELP, x))
@@ -312,7 +297,7 @@ def SetupToolBar(frame):
 			elif frame.ToolBarList[x].find("<DrScript>:") > -1:
 				try:
 					title = frame.ToolBarList[x][frame.ToolBarList[x].find("<DrScript>:")+11:]
-					i = frame.drscriptmenu.titles.index(title)					
+					i = frame.drscriptmenu.titles.index(title)
 					ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.drscriptmenu.ID_SCRIPT_BASE+i, x, title))
 				except:
 					pass
@@ -322,24 +307,12 @@ def SetupToolBar(frame):
 					ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_OTHER+x, x, title))
 					frame.Bind(wx.EVT_MENU, frame.OnToolBar, id=frame.ID_OTHER+x)
 				except:
-					pass		
+					pass
 			else:
 				ToolBarIdList.append(AddandReturn(frame, bitmapFileArray, frame.ID_OTHER+x, x))
 				frame.Bind(wx.EVT_MENU, frame.OnToolBar, id=frame.ID_OTHER+x)
 			x = x + 1
-			
-		if (len(frame.txtDocument.filename) < 1):
-			frame.toolbar.EnableTool(frame.ID_RELOAD, False)
-			frame.toolbar.EnableTool(frame.ID_RUN, False)
-			frame.toolbar.EnableTool(frame.ID_SET_ARGS, False)
-		
-		if (frame.txtPrompt.pid is -1):
-			frame.toolbar.EnableTool(frame.ID_END, False)
-		else:
-			frame.toolbar.EnableTool(frame.ID_RUN, False)
-			frame.toolbar.EnableTool(frame.ID_SET_ARGS, False)
-			frame.toolbar.EnableTool(frame.ID_PYTHON, False)
-								
+
 		frame.toolbar.Realize()
 	except:
 		drScrolledMessageDialog.ShowMessage(frame, "Error Loading the ToolBar.", "DrPython Error")
